<?php

namespace App\Factory;

use App\Exception;

class OrderFactory {
    
    private $connection;
    private $util;
    
    public function __construct (\Doctrine\DBAL\Connection $connection,
                                 \App\Util\Util $util) {

        $this->connection = $connection;
        $this->util       = $util;
    }
    
    public function insertSale ($sale, $generalParams, $order, $consumer) {
        $sql = "
            INSERT INTO VENDAREST
    			(CDFILIAL, NRVENDAREST, CDLOJA, NRMESA, CDVENDEDOR,
    			DTHRABERMESA, DTHRFECHMESA, CDOPERADOR, NRPESMESAVEN,
    			CDCLIENTE, CDCONSUMIDOR, IDPEDIDOPAGO)
    		VALUES
    			(:CDFILIAL, :NRVENDAREST, :CDLOJA, :NRMESA,
    			:CDVENDEDOR, TO_DATE(:CURRDATE, 'dd/mm/yyyy hh24:mi:ss'), null, :CDOPERADOR,
    			:NRPESMESAVEN, :CDCLIENTE, :CDCONSUMIDOR, 'S')";
    			
    	date_default_timezone_set($this->getCurrentTimezone($generalParams["SGESTADO"]));
    	
    	$params = [
    	    ":CDFILIAL"     => $generalParams["CDFILIAL"],
            ":NRVENDAREST"  => $sale,
            ":CDLOJA"       => $generalParams["CDLOJA"],
            ":NRMESA"       => "",
            ":CDVENDEDOR"   => $generalParams["CDVENDEDOR"],
            ":CURRDATE"     => date("d/m/Y H:i:s"),
            ":CDOPERADOR"   => $generalParams["CDOPERADOR"],
            ":NRPESMESAVEN" => "1",
            ":CDCLIENTE"    => $consumer["CDCLIENTE"],
            ":CDCONSUMIDOR" => $consumer["CDCONSUMIDOR"]    
    	];
    	
    	$this->connection->executeQuery($sql, $params);
    }
    
    public function insertSaleCommand ($generalParams, $order, $consumer, $deliveryFee) {
    	$sql = "
    		INSERT INTO COMANDAVEN 
    			(CDFILIAL, NRVENDAREST, NRCOMANDA, NRCOMANDAEXT, 
    			CDLOJA, DSCOMANDA, IDSTCOMANDA, SGSEXOCON, 
    			TXMOTIVCANCE, VRACRCOMANDA, CDPAIS, SGESTADO, 
    			CDMUNICIPIO, NRCEPCONSCOMAND, CDBAIRRO, DSBAIRRO,
    			NRENDECONSCOMAN, DSENDECONSCOMAN, DSCOMPLENDCOCOM,
    			DSREFENDCONSCOM, DSOBSCOMANDA, IDORGCMDVENDA, IDRETBALLOJA,
    			NRPEDMOBILE, NRLATPEDIDO, NRLONGPEDIDO, DSAREAATEND)
    		VALUES 
    			(:CDFILIAL, :NRVENDAREST, :NRCOMANDA, :NRCOMANDAEXT, 
    			:CDLOJA, :DSCOMANDA, :IDSTCOMANDA, :SGSEXOCON, 
    			:TXMOTIVCANCE, :VRACRCOMANDA, :CDPAIS, :SGESTADO, 
    			:CDMUNICIPIO, :NRCEPCONSCOMAND, :CDBAIRRO, :DSBAIRRO,
    			:NRENDECONSCOMAN, :DSENDECONSCOMAN, :DSCOMPLENDCOCOM,
    			:DSREFENDCONSCOM, :DSOBSCOMANDA, :IDORGCMDVENDA, :IDRETBALLOJA,
    			:NRPEDMOBILE, :NRLATPEDIDO, :NRLONGPEDIDO, :DSAREAATEND)";
    	
    	$params = [
    		":CDFILIAL"        => $generalParams["CDFILIAL"],
    		":NRVENDAREST"     => $order["NRVENDAREST"],
    		":NRCOMANDA"       => $order["NRCOMANDA"],
    		":NRCOMANDAEXT"    => $order["id"],
    		":CDLOJA"          => $generalParams["CDLOJA"],
    		":DSCOMANDA"       => "DLV_".$order["NRCOMANDA"],
    		":IDSTCOMANDA"     => $order["IDSTCOMANDA"],
    		":SGSEXOCON"       => "M",
    		":TXMOTIVCANCE"    => "",
    		":VRACRCOMANDA"    => $this->util->toFloat($deliveryFee),
    		":CDPAIS"          => "0055",
    		":SGESTADO"        => strtoupper($order["address"]["state"]), 
    		":CDMUNICIPIO"     => $consumer["CDMUNICIPIO"],
    		":NRCEPCONSCOMAND" => $order["address"]["zipCode"],
    		":CDBAIRRO"        => $consumer["CDBAIRRO"],
    		":DSBAIRRO"        => $consumer["DSBAIRRO"],
    		":NRENDECONSCOMAN" => (isset($order["address"]["number"]) && !empty($order["address"]["number"])) ? $order["address"]["number"] : "",
    		":DSENDECONSCOMAN" => substr($order["address"]["street"], 0, 44),
    		":DSCOMPLENDCOCOM" => $order["address"]["complement"],
    		":DSREFENDCONSCOM" => "",
    		":DSOBSCOMANDA"    => "",
    		":IDORGCMDVENDA"   => "DLV_PED",
    		":IDRETBALLOJA"    => ($order["order_type"] == "5") ? "N" : "S",
    		":NRPEDMOBILE"     => "PED".$order["id"],
    		":NRLATPEDIDO"     => (isset($order["address"]["geoLat"]) && !empty($order["address"]["geoLat"])) ? $order["address"]["geoLat"] : "",
    		":NRLONGPEDIDO"    => (isset($order["address"]["geoLon"]) && !empty($order["address"]["geoLon"])) ? $order["address"]["geoLon"] : "",
    		":DSAREAATEND"     => ""
    	];
    	
    	$this->connection->executeQuery($sql, $params);
    }
    
    public function insertCashMoviment ($order, $value, $generalParams, $consumer, $nsu) {
        $sql = "
            INSERT INTO MOVCAIXADLV
        		(CDFILIAL, NRVENDAREST, NRSEQMOVDLV, IDTIPOMOVIVEDLV,
        		VRMOVIVENDDLV, CDTIPORECE, CDCLIENTE, CDCONSUMIDOR,
        		NRCARTBANCODLV, CDNSUTEFDLV)
        	VALUES
        		(:CDFILIAL, :NRVENDAREST, :NRSEQMOVDLV, :IDTIPOMOVIVEDLV,
        		:VRMOVIVENDDLV, :CDTIPORECE, :CDCLIENTE, :CDCONSUMIDOR,
        		:NRCARTBANCODLV, :CDNSUTEFDLV)";

        $params = [
            ":CDFILIAL"        => $generalParams["CDFILIAL"],
	        ":NRVENDAREST"     => $order["NRVENDAREST"],
	        ":NRSEQMOVDLV"     => $order["NRSEQMOVDLV"],
	        ":IDTIPOMOVIVEDLV" => "E",
	        ":CDCLIENTE"       => $consumer["CDCLIENTE"],
	        ":CDCONSUMIDOR"    => $consumer["CDCONSUMIDOR"],
	        ":CDTIPORECE"      => $order["CDTIPORECE"],
	        ":VRMOVIVENDDLV"   => $this->util->toFloat($value),
	        ":NRCARTBANCODLV"  => null,
	        ":CDNSUTEFDLV"     => $nsu
        ];
        
        $this->connection->executeQuery($sql, $params);
    }
    
    public function isDuplicated ($generalParams, $order) {
        $sql = "
            SELECT C.NRPEDMOBILE

    		  FROM COMANDAVEN C

    		  JOIN VENDAREST V
    		    ON V.NRVENDAREST   = C.NRVENDAREST
    		   AND V.CDFILIAL      = C.CDFILIAL

    		 WHERE C.NRPEDMOBILE   = :NRPEDMOBILE
    		   AND V.CDFILIAL      = :CDFILIAL
    		   AND DTHRABERMESA    >= SYSDATE - 1
        ";
        
        $params = [
            ":CDFILIAL"    => $generalParams["CDFILIAL"],
            ":NRPEDMOBILE" => "PED".$order["id"]
        ];
        
        $result = $this->connection->fetchAssoc($sql, $params);
        
        return !empty($result);
    }
    
    

    /*
     * Função que busca dentro do json de TimeZones do projeto
     * o estado do cliente para buscar o Default Time Zone correto para o php
     * 
     */
    private function getCurrentTimezone ($state) {
        $timezones = array(
            "AC" => "America/Rio_branco",   "AL" => "America/Maceio",
            "AP" => "America/Belem",        "AM" => "America/Manaus",
            "BA" => "America/Bahia",        "CE" => "America/Fortaleza",
            "DF" => "America/Fortaleza",    "ES" => "America/Fortaleza",
            "GO" => "America/Fortaleza",    "MA" => "America/Fortaleza",
            "MT" => "America/Cuiaba",       "MS" => "America/Campo_Grande",
            "MG" => "America/Fortaleza",    "PR" => "America/Fortaleza",
            "PB" => "America/Fortaleza",    "PA" => "America/Belem",
            "PE" => "America/Recife",       "PI" => "America/Fortaleza",
            "RJ" => "America/Fortaleza",    "RN" => "America/Fortaleza",
            "RS" => "America/Fortaleza",    "RO" => "America/Porto_Velho",
            "RR" => "America/Boa_Vista",    "SC" => "America/Fortaleza",
            "SE" => "America/Maceio",       "SP" => "America/Fortaleza",
            "TO" => "America/Araguaia",    
        );
       
        return $timezones[$state];
    }
    
    public function validateOperator($generalParams) {
        $sql = "
            SELECT * FROM FILIOPER
             WHERE CDOPERADOR   = :CDOPERADOR
               AND CDFILIAL     = :CDFILIAL
        ";

        $params = [
            ":CDFILIAL"   => $generalParams["CDFILIAL"],
            ":CDOPERADOR" => $generalParams["CDOPERADOR"]
        ];
        
        return $this->connection->fetchAssoc($sql, $params);
    }
    
    public function fetchIsCashierSinc($filial) {
        $sql = "
            SELECT  IDSINCCAIXADLV 
              FROM  CAIXA
             WHERE  CDFILIAL              = :CDFILIAL
               AND  UPPER(IDSINCCAIXADLV) = 'S'
        ";

        $params = [
            ":CDFILIAL" => $filial
        ];

        return $this->connection->fetchAssoc($sql, $params);
    }
    
    //Prevents teknisa from crazy discount logics by partners ¯\_(ツ)_/¯
    public function validateOrder($cdFilial, $nrVendaRest, $orderId) {

        $taxSQL = "
            SELECT VRACRCOMANDA
              FROM COMANDAVEN
             WHERE NRVENDAREST      = :NRVENDAREST
               AND CDFILIAL         = :CDFILIAL
        ";

        $itemsSQL = "
            SELECT QTPRODCOMVEN, VRPRECCOMVEN, VRDESCCOMVEN
              FROM ITCOMANDAVEN
             WHERE NRVENDAREST      = :NRVENDAREST
               AND CDFILIAL         = :CDFILIAL
        ";

        $entradasSQL = "
            SELECT VRMOVIVENDDLV FROM MOVCAIXADLV
             WHERE NRVENDAREST      = :NRVENDAREST
               AND CDFILIAL         = :CDFILIAL
               AND IDTIPOMOVIVEDLV  = 'E'
        ";

        $saidasSQL = "
            SELECT VRMOVIVENDDLV FROM MOVCAIXADLV
             WHERE NRVENDAREST      = :NRVENDAREST
               AND CDFILIAL         = :CDFILIAL
               AND IDTIPOMOVIVEDLV  = 'S'
        ";
        
        $params = [
            ":CDFILIAL"     => $cdFilial,
            ":NRVENDAREST"  => $nrVendaRest
        ];

        $taxData      = $this->connection->fetchAssoc($taxSQL, $params);
        $itemsData    = $this->connection->fetchAll($itemsSQL, $params);
        $entradasData = $this->connection->fetchAll($entradasSQL, $params);
        $saidasData   = $this->connection->fetchAll($saidasSQL, $params);

        $taxSum      = isset($taxData['VRACRCOMANDA']) ? (string)$this->formatNumber($taxData['VRACRCOMANDA']) : "0.00";
        $itemsSum    = "0.00";
        $entradasSum = "0.00";
        $saidasSum   = "0.00";

        foreach($itemsData as $itemData) {
            $mul      = bcmul($this->formatNumber($itemData['VRPRECCOMVEN']), $this->formatNumber($itemData['QTPRODCOMVEN']), 2);
            $sub      = bcsub($mul, $this->formatNumber($itemData['VRDESCCOMVEN']), 2);
            $itemsSum = bcadd($itemsSum, $sub, 2);
        }

        foreach($entradasData as $entradaData) {
            $entradasSum = bcadd($entradasSum, $this->formatNumber($entradaData['VRMOVIVENDDLV']), 2);
        }

        foreach($saidasData as $saidaData) {
            $saidasSum = bcadd($saidasSum, $this->formatNumber($saidaData['VRMOVIVENDDLV']), 2);
        }

        $orderSum    = bcadd($itemsSum, $taxSum, 2);
        $paymentsSum = bcsub($entradasSum, $saidasSum, 2);

        if(bccomp($orderSum, $paymentsSum, 2) != 0) {
            $this->util->writeLog("execution", "", date("d/m/Y H:i:s")." Ocorreu divergência de valores no pedido #{$orderId} da filial {$cdFilial}.\n");
            throw new Exception\ParamsException("
                Não foi possível processar o pedido. Existe uma divergencia entre os itens do pedido e o valor pago.<br>
                Valor dos itens: {$orderSum}.<br>
                Valor pago: {$paymentsSum}.");
        }
    }
    
    private function formatNumber($number) {
        return str_replace(",", ".", $number);
    }
}
