<?php

namespace App\Factory;

use App\Exception;

class ProductFactory {
    
    private $connection;
    private $util;
    
    public function __construct (\Doctrine\DBAL\Connection $connection,
                                 \App\Util\Util $util) {
        $this->connection = $connection;
        $this->util = $util;
    }

    public function getProduct ($product) {
        $sql = "
            SELECT CDPRODUTO 
              FROM PRODEXTERNOINTE
              WHERE CDPRODINTE = :CDPRODEXTERNO
                AND CDPROJETO  = (SELECT CDPROJETO
                                   FROM (SELECT CDPROJETO
                                           FROM CADASTROPROJETO
                                          WHERE UPPER(DSPROJETO) = UPPER('PEDE PRONTO'))
                                WHERE ROWNUM = 1)
        ";

        $params = [
            ':CDPRODEXTERNO' => $product
        ];
        
        $extProduct = $this->connection->fetchAssoc($sql, $params);
        if(empty($extProduct))
            return null;
    
        $sql = "
            SELECT CDPRODUTO, CDARVPROD, NMPRODUTO, IDTIPOCOMPPROD
              FROM PRODUTO PR
             WHERE CDARVPROD  = :CDARVPROD
                OR CDPRODINTE = :CDPRODINTE
                OR CDPRODUTO  = :CDPRODUTO
        ";

        $productCode = str_replace(".", "", $extProduct['CDPRODUTO']);
        $productCode = preg_replace("/[^0-9A-Z]/", "", $productCode);

        $params = [
            ":CDARVPROD"  => $productCode,
            ':CDPRODINTE' => $productCode,
            ':CDPRODUTO'  => $productCode
        ];

        return $this->connection->fetchAssoc($sql, $params);
        
        //Verify if the product is linked to a product on the local database. Used for testing integrations.
        if(empty($productData)){
            if(file_exists(__DIR__ . "/../../productsInterface/externalProducts.json")){
                $externalProductsFile = file_get_contents(__DIR__ . "/../../productsInterface/externalProducts.json");
                $interfacedProducts   = json_decode($externalProductsFile, true);
                
                if(isset($interfacedProducts[$productCode]) && !empty($interfacedProducts[$product])){
                    $productCode = $interfacedProducts[$productCode];
                }
                
                $params = array(
                    ":CDARVPROD"  => $productCode,
                    ':CDPRODINTE' => $productCode,
                    ':CDPRODUTO'  => $productCode
                );
                
                $productData = $this->connection->fetchAll($sql, $params);
            }
        }
        
        return $productData;
    }
    
    public function getObservation ($observationCode) {

        $obs  = explode("|", $observationCode);

        $obs0 = preg_match("/[0-9a-zA-Z]{2}/", $obs[0]);
    	$obs1 = preg_match("/[0-9a-zA-Z]{2}/", $obs[1]);
    	
        if(strlen($obs[0]) != 2 || !$obs0)
    	    throw new Exception\ParamsException("
    	        O Código Grupo de Observação ({$obs[0]}) no código externo {$observationCode} deve possuir dois caracteres alfanuméricos, sem caracteres especiais.");
    	
    	if(strlen($obs[1]) != 2 || !$obs1)
    	    throw new Exception\ParamsException("
    	        O Código de Observação ({$obs[1]}), no código externo {$observationCode} deve possuir dois caracteres alfanuméricos, sem caracteres especiais.");
    	
        //===================== Busca por CDGRPOCOR =====================
    	
    	$sql = "
    		SELECT *
    		  FROM OCORRENCIA
    		 WHERE CDGRPOCOR = :CDGRPOCOR";

    	$params = array(
    		":CDGRPOCOR" => $obs[0]
    	);
    	
    	$cdgrpocor = $this->connection->fetchAssoc($sql, $params);
    	
    	if(!$cdgrpocor)
    	     throw new Exception\ParamsException("
    	        O código externo {$observationCode} não foi encontrado.<br>
    	        Verifique no Odhen Retail o código externo desta observação e faça a correção no portal Pede Pronto.");

    	//===================== Busca por CDOCOR =====================
    	
    	$sql = "
    		SELECT *
    		   FROM OCORRENCIA
    		  WHERE CDGRPOCOR = :CDGRPOCOR
    		    AND CDOCORR = :CDOCORR";

    	$params = array(
    	    ":CDGRPOCOR" => $obs[0],
    		  ":CDOCORR" => $obs[1]
    	);
    	
    	$cdocor = $this->connection->fetchAssoc($sql, $params);
    	if(!$cdocor)
    	     throw new Exception\ParamsException("
    	        O código externo {$observationCode} não foi encontrado.<br>
    	        Verifique no Odhen Retail o código externo desta observação e faça a correção no portal Pede Pronto.");
        
        return $cdocor;
    }
    
    public function insertProduct ($product) {
        $sql = "
            INSERT INTO ITCOMANDAVEN
    			(CDFILIAL, NRVENDAREST, NRCOMANDA, NRPRODCOMVEN,
    			CDPRODUTO, QTPRODCOMVEN, VRPRECCOMVEN, TXPRODCOMVEN,
    			IDSTPRCOMVEN, VRDESCCOMVEN, NRLUGARMESA, DTHRINCOMVEN,
    			IDPRODIMPFIS, CDLOJA, VRACRCOMVEN, NRSEQPRODCOM, NRSEQPRODCUP,
    			VRPRECCLCOMVEN, CDCAIXACOLETOR, CDPRODPROMOCAO, CDVENDEDOR, NRINSCRCONS, DSOBSPEDDIGCMD)
    		VALUES
    			(:CDFILIAL, :NRVENDAREST, :NRCOMANDA, :NRPRODCOMVEN,
    			:CDPRODUTO, :QTPRODCOMVEN, :VRPRECCOMVEN, :TXPRODCOMVEN,
    			:IDSTPRCOMVEN, :VRDESCCOMVEN, :NRLUGARMESA, SYSDATE,
    			:IDPRODIMPFIS, :CDLOJA, :VRACRCOMVEN, :NRSEQPRODCOM, :NRSEQPRODCUP,
    			:VRPRECCLCOMVEN, :CDCAIXACOLETOR, :CDPRODPROMOCAO, :CDVENDEDOR, :NRINSCRCONS, :DSOBSPEDDIGCMD)";
    		
	    date_default_timezone_set($this->getCurrentTimezone($product["SGESTADO"]));
	    
    	$params = array(
    	    ":CDFILIAL"       => $product["CDFILIAL"],
			":NRVENDAREST"    => $product["NRVENDAREST"],
			":NRCOMANDA"      => $product["NRCOMANDA"],
			":NRPRODCOMVEN"   => $product["NRPRODCOMVEN"],
			":CDPRODUTO"      => preg_replace("/[^0-9A-Z]/", "", $product["CDPRODUTO"]),
			":QTPRODCOMVEN"   => $product["QTPRODCOMVEN"],
			":VRPRECCOMVEN"   => $this->util->toFloat($product["VRPRECCOMVEN"]),
			":TXPRODCOMVEN"   => $product["TXPRODCOMVEN"],
			":IDSTPRCOMVEN"   => $product["IDSTPRCOMVEN"],
			":VRDESCCOMVEN"   => $this->util->toFloat($product["VRDESCCOMVEN"]),
			":NRLUGARMESA"    => $product["NRLUGARMESA"],
			":CURRDATE"       => date("d/m/Y H:i:s"),
			":IDPRODIMPFIS"   => $product["IDPRODIMPFIS"],
			":CDLOJA"         => $product["CDLOJA"],
			":VRACRCOMVEN"    => $this->util->toFloat($product["VRACRCOMVEN"]),
			":NRSEQPRODCOM"   => $product["NRSEQPRODCOM"],
			":NRSEQPRODCUP"   => $product["NRSEQPRODCUP"],
			":VRPRECCLCOMVEN" => $this->util->toFloat($product["VRPRECCLCOMVEN"]),
			":CDCAIXACOLETOR" => $product["CDCAIXACOLETOR"],
			":CDPRODPROMOCAO" => preg_replace("/[^0-9A-Z]/", "", $product["CDPRODPROMOCAO"]),
			":CDVENDEDOR"     => $product["CDVENDEDOR"],
			":NRINSCRCONS"    => $product["NRINSCRCONS"],
			":DSOBSPEDDIGCMD" => $product["DSOBSPEDDIGCMD"]
    	);
    	
    	$this->connection->executeQuery($sql, $params);
    }
    
    public function insertObservation ($obs) {
        $sql = "
            INSERT INTO	OBSITCOMANDAVEN
                (CDFILIAL, NRVENDAREST, NRCOMANDA, NRPRODCOMVEN, CDOCORR, CDGRPOCOR, NRPRODCOMVENOBS, QTPRODCOMVENOBS)
            VALUES
                (:CDFILIAL, :NRVENDAREST, :NRCOMANDA, :NRPRODCOMVEN, :CDOCORR, :CDGRPOCOR, :NRPRODCOMVENOBS, :QTPRODCOMVENOBS)";
		
	    $params = array(
	        ":CDFILIAL"        => $obs["CDFILIAL"],
            ":NRVENDAREST"     => $obs["NRVENDAREST"],
            ":NRCOMANDA"       => $obs["NRCOMANDA"],
            ":NRPRODCOMVEN"    => $obs["NRPRODCOMVEN"],
            ":CDOCORR"         => $obs["CDOCORR"],
            ":CDGRPOCOR"       => $obs["CDGRPOCOR"],
            ":NRPRODCOMVENOBS" => $obs["NRPRODCOMVENOBS"],
            ":QTPRODCOMVENOBS" => $obs["QTPRODCOMVENOBS"]  
        );
        
        $this->connection->executeQuery($sql, $params);
    }

    /*
     * Função que busca dentro do json de TimeZones do projeto
     * o estado do cliente para buscar o Default Time Zone correto para o php
     * 
     */
    private function getCurrentTimezone ($state) {
        $timezones = array(
            "AC" => "America/Rio_branco",   "AL" => "America/Maceio",
            "AP" => "America/Belem",        "AM" => "America/Manaus",
            "BA" => "America/Bahia",        "CE" => "America/Fortaleza",
            "DF" => "America/Fortaleza",    "ES" => "America/Fortaleza",
            "GO" => "America/Fortaleza",    "MA" => "America/Fortaleza",
            "MT" => "America/Cuiaba",       "MS" => "America/Campo_Grande",
            "MG" => "America/Fortaleza",    "PR" => "America/Fortaleza",
            "PB" => "America/Fortaleza",    "PA" => "America/Belem",
            "PE" => "America/Recife",       "PI" => "America/Fortaleza",
            "RJ" => "America/Fortaleza",    "RN" => "America/Fortaleza",
            "RS" => "America/Fortaleza",    "RO" => "America/Porto_Velho",
            "RR" => "America/Boa_Vista",    "SC" => "America/Fortaleza",
            "SE" => "America/Maceio",       "SP" => "America/Fortaleza",
            "TO" => "America/Araguaia",    
        );
       
        return $timezones[$state];
    }
    
}
